import os
import unittest
import threading
from pathlib import Path
from cetino.db.sqlite.type import SQLiteDataType
from cetino.db.sqlite.table_storage import SQLiteTableStorage

THREAD_COUNT = 100
RECORD_COUNT = 500

DB_FILE = 'test.db'
TABLE_NAME = 'test_table'


class TestStorage(SQLiteTableStorage):
    fields = {
        'name': SQLiteDataType.TEXT,
        'age': SQLiteDataType.INTEGER
    }
    table_name = TABLE_NAME


def threaded_insert(thread_id, db_file):
    storage_instance = TestStorage(db_file)
    data = [{'name': f'name_{thread_id}_{i}', 'age': i} for i in range(RECORD_COUNT)]

    with storage_instance:
        storage_instance.insert_many(data)


def threaded_query(db_file, expected_count=None):
    storage_instance = TestStorage(db_file)
    with storage_instance:
        n_records = len(storage_instance.query())
        if expected_count is not None:
            assert n_records == expected_count, f'expected {expected_count} records, got {n_records}'


class TestSQLiteTableStorage(unittest.TestCase):
    class TestStorage(SQLiteTableStorage):
        fields = {
            'name': SQLiteDataType.TEXT,
            'age': SQLiteDataType.INTEGER
        }
        table_name = TABLE_NAME

    def setUp(self):
        self.storage: SQLiteTableStorage = self.TestStorage(DB_FILE)
        """the test storage object"""
        with self.storage:
            self.storage.create()

    def tearDown(self):
        os.remove(DB_FILE)
        if Path('test.csv').exists():
            os.remove('test.csv')

    def test_validate_table_name(self):
        with self.assertRaises(ValueError):
            class InvalidStorage(SQLiteTableStorage):
                fields = {'name': SQLiteDataType.TEXT}
                table_name = ""

            InvalidStorage(DB_FILE)

    def test_validate_fields(self):
        with self.assertRaises(ValueError):
            class InvalidStorage(SQLiteTableStorage):
                fields = {}
                table_name = 'invalid_table'

            with InvalidStorage(DB_FILE):
                pass

    def test_validate_unique_fields(self):
        with self.assertRaises(ValueError):
            class InvalidStorage(SQLiteTableStorage):
                fields = {'name': SQLiteDataType.TEXT}
                table_name = 'invalid_table'
                unique_tuple = ('invalid_field',)

            InvalidStorage(DB_FILE)

    def test_validate_primary_key(self):
        with self.assertRaises(ValueError):
            class InvalidStorage(SQLiteTableStorage):
                fields = {'name': SQLiteDataType.TEXT}
                table_name = 'invalid_table'
                primary_key_tuple = ('invalid_pk',)

            InvalidStorage(DB_FILE)

    def test_insert_and_query(self):
        with self.storage:
            self.storage.insert({'name': 'Alice', 'age': 25})
            result = self.storage.query(use_pandas=False)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0]['name'], 'Alice')
        self.assertEqual(result[0]['age'], 25)

    def test_insert_many_and_query(self):
        with self.storage:
            data = [{'name': 'Alice', 'age': 25}, {'name': 'Bob', 'age': 30}]
            self.storage.insert_many(data)
            result = self.storage.query(use_pandas=False)
        self.assertEqual(len(result), 2)

    def test_query_dump(self):
        with self.storage:
            data = [{'name': 'Alice', 'age': 25}]
            self.storage.insert_many(data)
            self.storage.query_dump('test.csv')
        self.assertTrue(Path('test.csv').exists())


class MultiThreadedSQLiteTest(unittest.TestCase):

    def setUp(self):
        self.storage: SQLiteTableStorage = TestStorage(DB_FILE)
        with self.storage:
            self.storage.create()

    def tearDown(self):
        os.remove(DB_FILE)

    def insert_data(self):
        with self.storage:
            for _ in range(RECORD_COUNT):
                self.storage.insert({'name': 'Alice', 'age': 25})

    def query_data(self):
        with self.storage:
            self.storage.query()

    def test_multi_threaded_insert(self):
        db_file = DB_FILE

        threads = []

        # 创建多个线程，每个线程使用自己的SQLiteTableStorage实例
        for i in range(THREAD_COUNT):
            t = threading.Thread(target=threaded_insert, args=(i, db_file))
            threads.append(t)
            t.start()

        for t in threads:
            t.join()  # 等待所有线程完成

        # 使用主线程的storage实例检查结果
        with self.storage:
            result = self.storage.query(use_pandas=False)

        self.assertEqual(len(result), THREAD_COUNT * RECORD_COUNT)

    def test_multi_threaded_query(self):
        # First, insert some data
        with self.storage:
            for _ in range(RECORD_COUNT):
                self.storage.insert({'name': 'Alice', 'age': 25})

        # Second, launching multi-threading read
        threads = []
        for _ in range(THREAD_COUNT):
            t = threading.Thread(target=threaded_query, args=(DB_FILE, RECORD_COUNT))
            threads.append(t)
            t.start()

        for t in threads:
            t.join()



def connect(commit=False):
    """
    Decorator for managing connection with SQLite Database
    """

    def decorator(func):
        def wrapper(self, *args, **kwargs):
            # Before calling the decorated function, connect to database
            if self.is_connect is False:
                raise ConnectionError(
                    f"Connection with {self._db_file} is not established. You should use 'with' statement or manually manage the connection.")
            result = func(self, *args, **kwargs)
            # After calling the decorated function, commit and disconnect from database if commit is True
            if commit:
                self._commit()
            return result

        return wrapper

    return decorator



import sys
import logging

DEFAULT_LOG_FMT = "'%(asctime)s %(levelname)s %(message)s'"


def get_logger(name: str, log_path=None, log_fmt=DEFAULT_LOG_FMT, console=True):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)

    if log_path is not None:
        if not any(isinstance(hdlr, logging.FileHandler) and hdlr.baseFilename == str(log_path) for hdlr in logger.handlers):
            fh = logging.FileHandler(log_path)
            fh.setLevel(logging.INFO)
            fh.setFormatter(logging.Formatter(log_fmt))
            logger.addHandler(fh)

    if console:
        if not any(isinstance(hdlr, logging.StreamHandler) for hdlr in logger.handlers):
            ch = logging.StreamHandler(sys.stdout)  # set stream to stdout
            ch.setLevel(logging.INFO)
            ch.setFormatter(logging.Formatter(log_fmt))
            logger.addHandler(ch)

    return logger


def get_console_only_logger(name: str, log_fmt=DEFAULT_LOG_FMT):
    logger = get_logger(name, log_path=None, log_fmt=log_fmt, console=True)
    return logger


def file_only_logger(name: str, log_path, log_fmt=DEFAULT_LOG_FMT):
    logger = get_logger(name, log_path=log_path, log_fmt=log_fmt, console=False)
    return logger



import csv
import pandas as pd

from cetino.utils.io_utils import ensure_pathlib_path


def fields_match(func):
    """
    A decorator to ensure that the fields of the CSV file match the expected fields.
    """

    def wrapper(self, file_path, *args, **kwargs):
        file_path = ensure_pathlib_path(file_path)
        if not file_path.exists():
            return func(self, file_path, *args, **kwargs)

        with open(file_path, 'r') as f:
            reader = csv.DictReader(f)
            match = reader.fieldnames is None or reader.fieldnames == self.fields
            print("match: ", match)
            if not match:
                raise ValueError(
                    f"The fields in {file_path} do not match the expected fields. Expected: {self.fields}, Actual: {reader.fieldnames}")
        return func(self, file_path, *args, **kwargs)

    return wrapper


class CSVTableStorage:
    def __init__(self, fields, index_col=None):
        self.fields = fields
        self.index_col = index_col
        self._validate()

    @staticmethod
    def from_dict(record_dict: dict):
        """
        Create a CSVTableStorage from a dict

        :param record_dict: (dict) a dict with fields and index_col
        :return: (CSVTableStorage)
        """
        return CSVTableStorage(fields=list(record_dict.keys()))

    @staticmethod
    def from_dataframe(df: pd.DataFrame):
        """
        Create a CSVTableStorage from a dataframe

        :param df: (pd.DataFrame) a dataframe
        :return: (CSVTableStorage)
        """
        return CSVTableStorage(fields=df.columns.tolist(), index_col=df.index.name)

    @fields_match
    def read(self, file_path):
        """
        Read the csv file, return records

        :param file_path: (str) path to the csv file
        :return: (list<dict>)
        """
        with open(file_path, 'r') as f:
            reader = csv.DictReader(f)
            return [row for row in reader]

    @fields_match
    def read_to_df(self, file_path):
        """
        Read the csv file, return records

        :param file_path: (str) path to the csv file
        :return: (list<dict>)
        """
        record_df = pd.read_csv(file_path, index_col=self.index_col)
        self.fields = record_df.columns.tolist()
        return record_df

    @fields_match
    def write(self, file_path, records: list):
        """
        Write records to the csv file

        If the file does not exist, create it and write the records.
        Else, check if the fields are the same as the existing file, if not, raise ValueError.
        Else, append the records to the existing file.

        :param file_path: (str) path to the csv file
        :param records: (list<dict>)
        :return: None
        """
        file_path = ensure_pathlib_path(file_path)
        if not records or len(records) == 0:
            return
        if not file_path.exists():
            self._write_new_file(file_path, records)
        else:
            self._append_to_existing_file(file_path, records)

    @fields_match
    def write_from_df(self, file_path, record_df: pd.DataFrame):
        """
        Write records to the csv file

        If the file does not exist, create it and write the records.
        Else, check if the fields are the same as the existing file, if not, raise ValueError.
        Else, append the records to the existing file.

        :param file_path: (str) path to the csv file
        :param record_df: (pd.DataFrame) dataframe to write
        :return: None
        """
        # first convert record_df to list of dict
        records = record_df.to_dict(orient='records')
        self.write(file_path, records)

    @staticmethod
    def read_header(file_path):
        """
        Read the csv file, return header

        :param file_path: (str) path to the csv file
        :return: (list<dict>)
        """
        with open(file_path, 'r') as f:
            reader = csv.DictReader(f)
            return reader.fieldnames

    def _write_new_file(self, file_path, records):
        file_path = ensure_pathlib_path(file_path)
        with open(file_path, 'w') as f:
            writer = csv.DictWriter(f, fieldnames=self.fields)
            writer.writeheader()
            writer.writerows(records)

    def _append_to_existing_file(self, file_path, records):
        file_path = ensure_pathlib_path(file_path)
        with open(file_path, 'a') as f:
            writer = csv.DictWriter(f, fieldnames=self.fields)
            writer.writerows(records)

    def _validate(self):
        if self.fields is None or len(self.fields) == 0:
            raise ValueError("fields cannot be empty")

    def __str__(self):
        return f"CSVTableStorage(fields={self.fields}, index_col={self.index_col})"

    def __repr__(self):
        return self.__str__()



def add_quote(value):
    """
    Add quote to value if it is a string.

    e.g. "John" -> "\"John\""

    :param value: (str) value to be quoted
    :return: (str) quoted value
    """
    if isinstance(value, str):
        escaped_value = value.replace('"', '\\"')
        return f'"{escaped_value}"'
    else:
        return str(value)



import pathlib


def ensure_pathlib_path(path):
    """
    Ensure the path is a pathlib.Path object

    :param path: (str or pathlib.Path) the path
    :return: (pathlib.Path) the path
    """
    if path is None:
        return None
    if not isinstance(path, pathlib.Path):
        path = pathlib.Path(path)
    return path


def check_and_make_dir(dir_path):
    """
    Check if the directory exists, if not, create it

    :param dir_path: (str or pathlib.Path) the path to the directory
    :return: (pathlib.Path) the path to the directory
    """
    dir_path = ensure_pathlib_path(dir_path)
    if not dir_path.exists():
        dir_path.mkdir(parents=True, exist_ok=True)
    return dir_path


def ls(dir_path):
    """
    List all files in the directory

    :param dir_path: (str or pathlib.Path) the path to the directory
    :return: (list<str>) list of files
    """
    dir_path = ensure_pathlib_path(dir_path)
    return [str(file_path) for file_path in dir_path.iterdir()]


def pwd():
    """
    Get the current working directory

    :return: (pathlib.Path) the current working directory
    """
    return pathlib.Path.cwd()



from cetino.db.dialect import SQLDialect
from .sqlite.sql_builder import SQLiteSQLBuilder
from .mysql.sql_builder import MySQLSQLBuilder


_builder_dict = {
    SQLDialect.SQLITE: SQLiteSQLBuilder,
    SQLDialect.MYSQL: MySQLSQLBuilder
}


class SQLBuilder:
    supported_dialects = [SQLDialect.SQLITE, SQLDialect.MYSQL]

    def __init__(self, dialect: SQLDialect = SQLDialect.SQLITE):
        self.dialect = dialect
        self._validate()
        self._builder = self._get_builder()

    def _get_builder(self):
        if self.dialect in self.supported_dialects:
            return _builder_dict[self.dialect]()
        else:
            raise NotImplementedError(f"Dialect {self.dialect} is not supported yet.")

    def _validate(self):
        self._validate_dialect()

    def _validate_dialect(self):
        if self.dialect not in SQLDialect:
            raise ValueError(f"Dialect {self.dialect} is not supported.")


__all__ = ["SQLBuilder"]



import uuid
import logging
import sqlite3
from sqlite3 import Error

from ._decorator import connect
from cetino.utils.io_utils import ensure_pathlib_path
from cetino.utils.log_utils import get_console_only_logger, get_logger


class SQLiteStorage:
    def __init__(self, data_path, log_path=None):
        """
        Initialization.
        - Create or attach to a sqlite database file.
        - Create a single connection for reuse.(SQLite only support single-thread operation)

        :param data_path: (str | pathlib.Path) database file path
        :param log_path: (str | pathlib.Path | None) log file path, if None, only print to console
        :return None
        """
        self._uuid = str(uuid.uuid4())
        self._logger = self.__configure_logger(log_path)
        self._db_file = ensure_pathlib_path(data_path)
        self._conn = None
        """Single connection for reuse."""
        self.is_connect = False
        """State variable to indicate whether the connection is established."""

    @property
    def db_file(self):
        return self._db_file

    @connect()
    def execute(self, sql: str):
        """Execute SQL statement."""
        if self._conn is None:
            raise ConnectionError(
                f"Connection with {self._db_file} is not established. You should use 'with' statement or manually manage the connection.")

        c = self._conn
        try:
            result = c.execute(sql)
            self._log(f'Execute SQL: {sql}', level=logging.INFO)
            return result
        except Error as e:
            self._log(f'Execute SQL: {sql} failed', level=logging.ERROR)
            self._log(str(e), level=logging.ERROR)
            raise e  # 直接抛出原始的 error

    def _connect(self):
        if self.is_connect is False:
            self._log(f'Establishing connection with {self._db_file}...')
            self._conn = sqlite3.connect(self._db_file)
            self.is_connect = True
            self._log(f'Connection with {self._db_file} established', level=logging.INFO)
        else:
            self._log(f'Connection with {self._db_file} already established', level=logging.INFO)

    def _commit(self):
        self._conn.commit()
        self._log(f'Commit changes to {self._db_file}', level=logging.INFO)

    def _disconnect(self):
        if self.is_connect is True:
            self._log(f'Closing connection with {self._db_file}...')
            self._conn.close()
            self.is_connect = False
            self._log(f'Connection with {self._db_file} closed', level=logging.INFO)
        else:
            self._log(f'Connection with {self._db_file} already closed', level=logging.INFO)

    def _log(self, message: str, level=logging.INFO, **kwargs):
        self._logger.log(level, message, **kwargs)

    def __configure_logger(self, log_path):
        if log_path is None:
            logger = get_console_only_logger(self._logger_name)
        else:
            logger = get_logger(self._logger_name, log_path=log_path, console=False)
        return logger

    @property
    def _logger_name(self):
        return f"{self.__class__.__name__}_{self._uuid}_logger"

    def __str__(self):
        return f"{self.__class__.__name__}({self._db_file})"

    def __repr__(self):
        return self.__str__()

    def __enter__(self):
        self._connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._disconnect()


__all__ = ["SQLiteStorage"]



import abc
import logging
import pandas as pd

from ._decorator import connect
from .storage import SQLiteStorage
from .sql_builder import SQLiteSQLBuilder
from cetino.fs.csv.storage import CSVTableStorage
from .type import SQLiteDataType

default_primary_key_tuple = ('_id',)


class SQLiteTableStorage(SQLiteStorage, abc.ABC):
    """
    SQLite Table Manipulation, DO NOT instantiate this class directly.

    You have to override the following properties to define the table metadata:
    - fields: (dict) list of fields, e.g. {'name': SQLiteDataType.TEXT, 'age': SQLiteDataType.INTEGER}
    - table_name: (str) table name, e.g. 'employees'
    - (optional) primary_key_tuple: (tuple) primary key, e.g. ('id'), if not override, return default primary key
    - (optional) unique_tuple: (tuple) unique fields, e.g. ('name', 'age')
    """
    primary_key_tuple = default_primary_key_tuple
    unique_tuple = ()

    @property
    @abc.abstractmethod
    def fields(self) -> dict:
        """list of fields, e.g. {'name': SQLiteDataType.TEXT, 'age': SQLiteDataType.INTEGER}"""
        pass

    @property
    @abc.abstractmethod
    def table_name(self) -> str:
        """table name, e.g. 'employees'"""
        pass

    @property
    def field_names_list(self):
        return list(self.fields.keys())

    def __init__(self, data_path, log_path=None):
        """
        :param data_path: (str | pathlib.Path) database file path
        :param log_path: (str | pathlib.Path | None) log file path, if None, only print to console
        """
        super().__init__(data_path, log_path)
        self.ddl = None
        self._validate_table_metadata()

    @connect()
    def create(self, allow_exist: bool = False):
        """
        Create table.
        :return: None
        """
        full_fields = self.fields.copy()
        if self.primary_key_tuple == default_primary_key_tuple:
            for pk_item in self.primary_key_tuple:
                full_fields[pk_item] = SQLiteDataType.INTEGER
        self.ddl = SQLiteSQLBuilder.create_table_sql(table_name=self.table_name, fields_dict=full_fields,
                                                     primary_key_tuple=self.primary_key_tuple,
                                                     unique_tuple=self.unique_tuple,
                                                     allow_exist=allow_exist)
        self.execute(self.ddl)
        self._log(f'Table {self.table_name} created', level=logging.INFO)

    @connect()
    def drop(self, allow_not_exist=True):
        """
        Drop table.
        :return: None
        """
        sql = SQLiteSQLBuilder.drop_table_sql(self.table_name, allow_not_exist=allow_not_exist)
        self.execute(sql)
        self._log(f'Table {self.table_name} dropped', level=logging.INFO)

    @connect()
    def query(self, limit: int = None, offset: int = None, use_pandas: bool = False):
        """
        Get records with full fields from table.

        :param limit: (int)
        :param offset: (int)
        :param use_pandas: (bool) if True, return pandas.DataFrame, else return list of dict
        :return: (list<dict> | pd.DataFrame)
        """
        sql = SQLiteSQLBuilder.query_sql(table_name=self.table_name, fields_list=self.field_names_list,
                                         limit=limit, offset=offset)
        records = self.execute(sql)
        records = [dict(zip([field[0] for field in records.description], record)) for record in records]
        if use_pandas:
            records = pd.DataFrame(records)
            records.set_index(self.primary_key_tuple, inplace=True)
        return records

    @connect()
    def query_dump(self, save_path, limit: int = None, offset: int = None):
        """
        Dump records with full fields from table to csv file.

        :param save_path: (str | pathlib.Path) csv file path
        :param limit: (int)
        :param offset: (int)
        :return: None
        """
        records = self.query(limit=limit, offset=offset, use_pandas=False)
        csv_storage = self.get_csv_storage()
        self._log(f'Dump {len(records)} records to {save_path}...', level=logging.INFO)
        csv_storage.write(save_path, records)
        self._log(f'Dumping finished', level=logging.INFO)

    @connect(commit=True)
    def insert(self, record: dict):
        insert_sql = SQLiteSQLBuilder.insert_sql(table_name=self.table_name, data_dict_list=[record])
        cursor = self.execute(insert_sql)
        return cursor.rowcount

    @connect(commit=True)
    def insert_many(self, record_list: list):
        insert_sql = SQLiteSQLBuilder.insert_sql(table_name=self.table_name, data_dict_list=record_list)
        cursor = self.execute(insert_sql)
        return cursor.rowcount

    @connect(commit=True)
    def empty(self):
        delete_sql = SQLiteSQLBuilder.delete_sql(table_name=self.table_name)
        print(f"delete_sql: {delete_sql}")
        cursor = self.execute(delete_sql)
        return cursor.rowcount

    def get_csv_storage(self):
        return CSVTableStorage(fields=list(self.fields.keys()), index_col=self.primary_key_tuple)

    def _validate_table_metadata(self):
        self._validate_table_name()
        self._validate_fields()
        self._validate_unique_fields()
        self._validate_primary_key()

    def _validate_table_name(self):
        if not self.table_name:
            msg = 'Table name is not defined.'
            self._log(msg, level=logging.ERROR)
            raise ValueError(msg)

    def _validate_fields(self):
        if not self.fields or not isinstance(self.fields, dict) or len(self.fields) == 0:
            msg = 'Fields must be defined as a non-empty dictionary.'
            self._log(msg, level=logging.ERROR)
            raise ValueError(msg)

    def _validate_unique_fields(self):
        if self.unique_tuple:
            if not isinstance(self.unique_tuple, (list, tuple)):
                msg = 'The "unique_tuple" attribute must be a list or tuple.'
                self._log(msg, level=logging.ERROR)
                raise ValueError(msg)
            for field in self.unique_tuple:
                if field not in self.fields:
                    msg = f'Unique field "{field}" is not defined in the fields.'
                    self._log(msg, level=logging.ERROR)
                    raise ValueError(msg)

    def _validate_primary_key(self):
        # 1. primary key tuple cannot be empty
        if not self.primary_key_tuple or not isinstance(self.primary_key_tuple, (list, tuple)):
            msg = '`primary_key_tuple` must be a non-empty list or tuple.'
            self._log(msg, level=logging.ERROR)
            raise ValueError(msg)
        if self.primary_key_tuple != default_primary_key_tuple:
            # 2. user specified primary key must be defined in the fields
            for pk_item in self.primary_key_tuple:
                if pk_item not in self.fields:
                    msg = f'User specified primary key "{pk_item}" is not defined in the fields.'
                    self._log(msg, level=logging.ERROR)
                    raise ValueError(msg)


__all__ = ['SQLiteTableStorage']



class BaseSQLBuilder:
    _select_clause_sep = "\n"
    _declare_clause_sep = ",\n"
    _field_declare_sep = ",\n"



    @staticmethod
    def primary_key_sql(primary_key_tuple):
        return f'PRIMARY KEY ({", ".join(primary_key_tuple)})'

    @classmethod
    def fields_declare_sql(cls, fields_dict):
        if not isinstance(fields_dict, dict):
            raise TypeError(f"fields_dict must be a dict({{[FIELD_NAME]:[FIELD_TYPE]}}), got {type(fields_dict)}")
        fields_str = cls._field_declare_sep.join([f'{field} {(fields_dict[field]).value}' for field in fields_dict])
        return fields_str

    @staticmethod
    def unique_sql(unique_tuple):
        if not unique_tuple or len(unique_tuple) == 0:
            return ''
        else:
            return f'UNIQUE ({", ".join(unique_tuple)})'

    @staticmethod
    def where_clause(cond_list: list):
        """
        Generate where clause.

        :param cond_list: (list) where clause, e.g. ['age > 20', 'name = "John"']
        :return: (str) where clause, e.g. 'WHERE age > 20 AND name = "John"'
        """
        return f'WHERE {" AND ".join(cond_list)}' if cond_list else ''

    @staticmethod
    def order_by_clause(order_by_dict: dict):
        """
        Generate order by clause.

        :param order_by_dict: (dict) order by clause, e.g. {'name': 'ASC', 'age': 'DESC'}
        :return: (str) order by clause, e.g. 'ORDER BY name ASC, age DESC'
        """
        return f'ORDER BY {", ".join([f"{field} {order_by_dict[field]}" for field in order_by_dict])}' if order_by_dict else ''

    @staticmethod
    def limit_clause(limit: int):
        return f'LIMIT {limit}' if limit else ''

    @staticmethod
    def offset_clause(offset: int):
        return f'OFFSET {offset}' if offset else ''


__all__ = ['BaseSQLBuilder']



import unittest
from enum import Enum

from cetino.db.sqlite.sql_builder import BaseSQLBuilder, SQLiteSQLBuilder


class FieldType(Enum):
    TEXT = "TEXT"
    INTEGER = "INTEGER"


class TestBaseSQLBuilder(unittest.TestCase):

    def test_primary_key_sql(self):
        result = BaseSQLBuilder.primary_key_sql(("id", "name"))
        print(f"[test_primary_key_sql] result: {result}, expected: PRIMARY KEY (id, name)")
        self.assertEqual(result, "PRIMARY KEY (id, name)")

    def test_fields_declare_sql(self):
        fields = {"name": FieldType.TEXT, "age": FieldType.INTEGER, "id": FieldType.INTEGER}
        result = BaseSQLBuilder.fields_declare_sql(fields)
        print(f"[test_fields_declare_sql] result: {result}, expected: name TEXT,\nage INTEGER,\nid INTEGER")
        self.assertEqual(result, "name TEXT,\nage INTEGER,\nid INTEGER")

    def test_unique_sql(self):
        result = BaseSQLBuilder.unique_sql(("name",))
        print(f"[test_unique_sql] result: {result}, expected: UNIQUE (name)")
        self.assertEqual(result, "UNIQUE (name)")

    def test_where_clause(self):
        conditions = ['age > 20', 'name = "John"']
        result = BaseSQLBuilder.where_clause(conditions)
        print(f"[test_where_clause] result: {result}, expected: WHERE age > 20 AND name = \"John\"")
        self.assertEqual(result, 'WHERE age > 20 AND name = "John"')


class TestSQLiteSQLBuilder(unittest.TestCase):

    def test_query_sql(self):
        table_name = "users"
        fields = ["name", "age"]
        conditions = ['age > 20']
        order_by = {"age": "DESC"}
        limit = 10
        result = SQLiteSQLBuilder.query_sql(table_name, fields, conditions, order_by, limit)
        print(f"[test_query_sql] result: {result}, expected: SELECT name, age\nFROM users\nWHERE age > 20\nORDER BY age DESC\nLIMIT 10;")
        expected_sql = 'SELECT name, age\nFROM users\nWHERE age > 20\nORDER BY age DESC\nLIMIT 10;'
        self.assertEqual(result, expected_sql)

    def test_insert_sql(self):
        table_name = "users"
        data = [{"name": "John", "age": 25}, {"name": "Jane", "age": 30}]
        result = SQLiteSQLBuilder.insert_sql(table_name, data)
        print(f"[test_insert_sql] result: {result}, expected: INSERT INTO users (name, age) VALUES (\"John\", 25), (\"Jane\", 30);")
        expected_sql = 'INSERT INTO users (name, age) VALUES ("John", 25), ("Jane", 30);'
        self.assertEqual(result, expected_sql)

    def test_create_table_sql(self):
        table_name = "users"
        fields = {"name": FieldType.TEXT, "age": FieldType.INTEGER}
        primary_key = ("id",)
        unique = ("name",)
        result = SQLiteSQLBuilder.create_table_sql(table_name, fields, primary_key, unique)
        print(f"[test_create_table_sql] result:\n{result}, expected:\nCREATE TABLE IF NOT EXISTS users (\nname TEXT,\nage INTEGER,\nPRIMARY KEY (id),\nUNIQUE (name)\n);")
        expected_sql = '''CREATE TABLE IF NOT EXISTS users (
name TEXT,
age INTEGER,
PRIMARY KEY (id),
UNIQUE (name)
);'''
        self.assertEqual(result, expected_sql)


if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages

with open("requirements.txt", "r") as f:
    requirements = f.read().splitlines()

setup(
    name='cetino',
    version='0.1.3',
    packages=find_packages(),
    install_requires=requirements,
    author='steveflyer',
    author_email='steveflyer7@gmail.com',
    description='Store and load your data in a unified and light-weight way.',
    license='MIT',
    keywords='sqlite dict storage',
    url='https://github.com/stevieflyer/cetino.git'
)



import enum


class PostgreDataType(enum.Enum):
    INT2 = "INT2"
    INT4 = "INT4"
    INT8 = "INT8"
    FLOAT4 = "FLOAT4"
    FLOAT8 = "FLOAT8"
    CHAR = "CHAR"
    VARCHAR = "VARCHAR"
    TEXT = "TEXT"



import enum


class MySQLDataType(enum.Enum):
    TINYINT = "TINYINT"
    SMALLINT = "SMALLINT"
    MEDIUMINT = "MEDIUMINT"
    INT = "INT"
    BIGINT = "BIGINT"
    FLOAT = "FLOAT"
    DOUBLE = "DOUBLE"
    CHAR = "CHAR"
    VARCHAR = "VARCHAR"
    TEXT = "TEXT"
    TINYTEXT = "TINYTEXT"
    MEDIUMTEXT = "MEDIUMTEXT"
    LONGTEXT = "LONGTEXT"



from .._base import BaseSQLBuilder
from cetino.utils.string_utils import add_quote


class SQLiteSQLBuilder(BaseSQLBuilder):
    @classmethod
    def query_sql(cls, table_name: str, fields_list=None, cond_list=None, order_by_dict=None, limit=None, offset=None):
        fields_str = ', '.join(fields_list) if fields_list else "*"
        where_clause = cls.where_clause(cond_list)
        order_by_clause = cls.order_by_clause(order_by_dict)
        limit_clause = cls.limit_clause(limit)
        offset_clause = cls.offset_clause(offset)
        clauses = cls._select_clause_sep.join(
            [c for c in [where_clause, order_by_clause, limit_clause, offset_clause] if c != ""])
        sql = f"""
SELECT {fields_str}
FROM {table_name}
{clauses};"""
        return sql.strip()

    @classmethod
    def insert_sql(cls, table_name: str, data_dict_list: list):
        if not data_dict_list or len(data_dict_list) == 0:
            raise ValueError(f"data_dict_list must be a list of dict, got {data_dict_list}")
        values_list = [[add_quote(value) for value in data_dict.values()] for data_dict in data_dict_list]
        values_str = ', '.join([f'({", ".join(values)})' for values in values_list])
        insert_sql = f"""INSERT INTO {table_name} ({', '.join(data_dict_list[0].keys())}) VALUES {values_str};"""
        return insert_sql.strip()

    @classmethod
    def create_table_sql(cls, table_name: str, fields_dict, primary_key_tuple=None, unique_tuple=None, allow_exist=False):
        allow_exist_clause = "IF NOT EXISTS " if allow_exist else ""
        fields_declare = cls.fields_declare_sql(fields_dict)
        primary_key_declare = cls.primary_key_sql(primary_key_tuple) if primary_key_tuple else ""
        unique_declare = cls.unique_sql(unique_tuple) if unique_tuple else ""

        clauses = [fields_declare, primary_key_declare, unique_declare]
        sql_clauses = cls._declare_clause_sep.join(filter(bool, clauses))

        sql = f"""
CREATE TABLE {allow_exist_clause}{table_name} (
{sql_clauses}
);"""
        return sql.strip()

    @classmethod
    def drop_table_sql(cls, table_name: str, allow_not_exist=False):
        allow_not_exist_clause = "IF EXISTS " if allow_not_exist else ""
        sql = f"""DROP TABLE {allow_not_exist_clause}{table_name};"""
        return sql.strip()

    @classmethod
    def delete_sql(cls, table_name, cond_list=None):
        where_clause = cls.where_clause(cond_list)
        sql = f"""
DELETE FROM {table_name}
{where_clause};"""
        return sql.strip()


__all__ = ["SQLiteSQLBuilder"]
